
/* std.h */

#ifndef __PROG_STD_H__
#define __PROG_STD_H__

/* configuration */

// #define USE_OWN_DICT
// #define USE_COMPLETE_DICT

// #define USE_ROOT_TSTRING
// #define USE_ROOT_TSYSTEM
// #define USE_FIND_FIRST

/* assert */

#include <cassert>

/* string */

#ifdef USE_ROOT_TSTRING
   #include "TString.h"
#else
   #include <string>
#endif

/* std namespace */

#if __GNUC__ == 2
   #define GCC2
   using namespace std;
#endif

#ifndef GCC2
   using namespace std;
#endif

/* hash namespace */

#undef MAP_TYPE
#undef SET_TYPE

#undef MAP_INCL
#undef SET_INCL

#undef USE_STR_HASH

#if __GNUC__ == 2

   /* GCC 2 ... without namespace and without subdirectory */

   #define MAP_TYPE hash_map
   #define SET_TYPE hash_set

   #define MAP_INCL <hash_map>
   #define SET_INCL <hash_set>

   #define USE_STR_HASH

#endif

#if __GNUC__ == 3

   /* GCC 3 ... __gnu_cxx namespace and ext subdirectory */

   #define MAP_TYPE __gnu_cxx::hash_map
   #define SET_TYPE __gnu_cxx::hash_set

   #define MAP_INCL <ext/hash_map>
   #define SET_INCL <ext/hash_set>

   #define USE_STR_HASH

#endif

#if  __GNUC__ >= 4

   /* GCC 4.0, 4.1, 4.2 ... native hash map */

   #if __GNU__C == 4 && __GNUC_MINOR__ <= 2
      #define MAP_TYPE __gnu_cxx::hash_map
      #define SET_TYPE __gnu_cxx::hash_set

      #define MAP_INCL <ext/hash_map>
      #define SET_INCL <ext/hash_set>

      #define USE_STR_HASH

      // #undef __DEPRECATED
   #endif

   #if defined (__GXX_EXPERIMENTAL_CXX0X) || __cplusplus >= 201103L
      #define MAP_TYPE std::unordered_map
      #define SET_TYPE std::unordered_set

      #define MAP_INCL <unordered_map>
      #define SET_INCL <unordered_set>
   #endif

#endif

#ifdef _MSC_VER
   #define MAP_TYPE hash_map
   #define SET_TYPE hash_set

   #define USE_STR_HASH
#endif

/* default values */

#ifndef MAP_TYPE
   #define MAP_TYPE map
#endif

#ifndef SET_TYPE
   #define SET_TYPE set
#endif

#ifndef MAP_INCL
   #define MAP_INCL <map>
#endif

#ifndef SET_INCL
   #define SET_INCL <set>
#endif

/* namespace */

#define NAMESPACE finch
#define NAMESPACE_STRING "finch"
// modify also cmm/cmm.g

#ifdef NAMESPACE
   #define OPEN_NAMESPACE namespace NAMESPACE {
   #define CLOSE_NAMESPACE } // end of namespace
   #define USE_NAMESPACE using namespace NAMESPACE;
#else
   #define OPEN_NAMESPACE
   #define CLOSE_NAMESPACE
   #define USE_NAMESPACE
#endif

/* root string */

#ifdef USE_ROOT_TSTRING

OPEN_NAMESPACE

class string : public TString
{
   public:
      typedef Ssiz_t size_type;
      static const size_type npos = kNPOS;

      const char * c_str () const { return Data (); }
      size_type length () const  { return Length (); }

      string substr (size_type start, size_type len) const
      {
         // 'len' characters from 'start' position
         return TString::operator() (start, len);
      }

      string substr (size_type start) const
      {
         // 'len' characters from 'start' position
         return substr (start, Length () - start);
      }

      size_type find (char c) const
      {
         return Index (c);
      }

      size_type rfind (char c) const
      {
         size_type inx = Length ();
         const char * txt = Data ();

         while (inx > 1 && txt [inx-1] != c)
            inx --;

         if (inx == 0)
            return npos;
         else
            return inx-1;
      }

      size_type find (string c) const
      {
         return Index (c);
      }

      void reserve (size_type size) { AdjustCapacity (size); }
      void append (const char * s, size_type size) { Append (s, size); }

      operator std::string () const
      {
         return Data ();
      }

      string () : TString () { }
      string (char c) : TString (c) { }
      string (int n, char c) : TString (c, n) { } // !?
      string (const char *s, int n) : TString (s, n) { } // !?
      string (const char *s) : TString (s) { }
      string (const std::string & s) : TString (s) { }
      string (const TSubString &s) : TString (s) { }
      string (const TString & s) : TString (s) { }
      string (const string & s) : TString (s) { }
};

CLOSE_NAMESPACE

#endif

#endif /* __PROG_STD_H__ */
